/*
 * Musepack audio compression
 * Copyright (C) 1999-2004 Buschmann/Klemm/Piecha/Wolf
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once



typedef struct
{
	mpc_uint32_t	bitbuf;				// bitbuffer
	mpc_uint8_t		*buf;				// byte buffer
	int				bits;				// number of bits in bitbuffer
} mpc_bitstream;


static mpc_inline void mpc_bs_init(mpc_bitstream *bs, mpc_uint8_t *b)
{
	bs->bitbuf = 0;
	bs->buf    = b;
	bs->bits   = 0;
}

static mpc_inline void mpc_bs_putbits(mpc_bitstream *bs, mpc_uint32_t val, int num)
{
	bs->bits += num;
	if (num < 32) val &= (1<<num)-1;

	if (bs->bits > 32) {
		bs->bits -= 32;
		bs->bitbuf |= val >> (bs->bits);
		*bs->buf++ = ( bs->bitbuf >> 24 ) & 0xff;
		*bs->buf++ = ( bs->bitbuf >> 16 ) & 0xff;
		*bs->buf++ = ( bs->bitbuf >>  8 ) & 0xff;
		*bs->buf++ = ( bs->bitbuf >>  0 ) & 0xff;
		bs->bitbuf = val << (32 - bs->bits);
	} else
	bs->bitbuf |= val << (32 - bs->bits);
}

static mpc_inline void mpc_bs_writebits(mpc_bitstream *bs)
{
	while (bs->bits >= 8) {
		*bs->buf++ = (bs->bitbuf >> 24) & 0xff;
		bs->bitbuf <<= 8;
		bs->bits -= 8;
	}
}

static mpc_inline void mpc_bs_bytealign(mpc_bitstream *bs, int digit)
{
	int	to_write = (8 - bs->bits & 0x07) & 0x07;
	if (to_write > 0) {
		mpc_bs_putbits(bs, (1 << to_write) - digit, to_write);
	}
}


void mpc_bs_encode_size(mpc_bitstream *bs, mpc_uint64_t size, mpc_bool_t add_code_size);
void mpc_bs_encode_enum(mpc_bitstream *bs, const mpc_uint32_t bits, const mpc_uint_t N);
void mpc_bs_encode_log(mpc_bitstream *bs, mpc_uint32_t value, mpc_uint32_t max);
void mpc_bs_encode_golomb(mpc_bitstream *bs, mpc_uint32_t nb, mpc_uint_t k);



