/*
 * Musepack audio compression
 * Copyright (C) 1999-2004 Buschmann/Klemm/Piecha/Wolf
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once


// FIXME : define this somewhere else
#ifndef NULL
#define NULL 0
#endif

#define MPC_FRAME_LENGTH (36 * 32)
#define MAX_FRAME_SIZE 4352



typedef struct {
	mpc_uint16_t		Code;        // >= 14 bit
	mpc_uint16_t		Length;      // >=  4 bit
} Huffman_t;

typedef struct {

	// bitstream
	mpc_bitstream		bs;
	mpc_uint8_t			*buffer;				// Buffer for audio packets

	mpc_uint64_t		outputBits;				// Counter for the number of written bits in the bitstream

	// total stats
	mpc_uint64_t		samples_encoded;		// total number of encoded (and delivered) samples
	mpc_uint64_t		samples_to_skip;		// number of samples to skip at start

	mpc_uint_t			framesInBlock;			// Number of frames in current block
	mpc_uint_t			frames_per_block_pwr;	// Number of frame in a block = 1 << frames_per_block_pwr

	// seeking
	mpc_uint32_t		*seek_table;
	mpc_uint32_t		seek_pos;				/// current position in the seek table
	mpc_uint32_t		seek_ref;				/// reference position for the seek information
	mpc_uint32_t		seek_ptr;				/// position of the seek pointer block
	mpc_uint32_t		seek_pwr;				/// keep a seek table entry every 2^seek_pwr block
	mpc_uint32_t		block_cnt;				/// number of encoded blocks


	mpc_uint32_t		MS_Channelmode;
	mpc_uint32_t		Overflows;				//  = 0;      // number of internal (filterbank) clippings
	mpc_uint32_t		MaxBand;				/// number of non zero bands in last frame

 	mpc_int32_t			SCF_Index_L [32] [3];
	mpc_int32_t			SCF_Index_R [32] [3];	// holds scalefactor-indices
	mpc_int32_t			SCF_Last_L  [32];
	mpc_int32_t			SCF_Last_R  [32];		// Last coded SCF value
	mpc_quantizer		Q [32];					// holds quantized samples
	mpc_int32_t			Res_L [32];
	mpc_int32_t			Res_R [32];				// holds the chosen quantizer for each subband
	mpc_bool_t			DSCF_Flag_L [32];
	mpc_bool_t			DSCF_Flag_R [32];		// differential SCF used?
	mpc_bool_t			MS_Flag[32];			// MS used?

	// configuration
	mpc_encoder_config_t	config;

	// psy model in use
	PsyModel				psy_model;

} mpc_enc_t;




